@@ -70,6 +70,7 @@ enum {
 	CHILD_REMOVED,
 	CHILD_MOVED,
 	REALIZED,
+	SHOW_TO_USER,
 	LAST_SIGNAL
 };
 
@@ -96,6 +97,7 @@ static void get_property (GObject * obj, guint id, GValue * value, GParamSpec *
 static void g_value_transform_STRING_BOOLEAN (const GValue * in, GValue * out);
 static void g_value_transform_STRING_INT (const GValue * in, GValue * out);
 static void handle_event (DbusmenuMenuitem * mi, const gchar * name, const GValue * value, guint timestamp);
+static void send_about_to_show (DbusmenuMenuitem * mi, void (*cb) (DbusmenuMenuitem * mi, gpointer user_data), gpointer cb_data);
 
 /* GObject stuff */
 G_DEFINE_TYPE (DbusmenuMenuitem, dbusmenu_menuitem, G_TYPE_OBJECT);
@@ -113,6 +115,7 @@ dbusmenu_menuitem_class_init (DbusmenuMenuitemClass *klass)
 	object_class->get_property = get_property;
 
 	klass->handle_event = handle_event;
+	klass->send_about_to_show = send_about_to_show;
 
 	/**
 		DbusmenuMenuitem::property-changed:
@@ -211,6 +214,22 @@ dbusmenu_menuitem_class_init (DbusmenuMenuitemClass *klass)
 	                                           NULL, NULL,
 	                                           _dbusmenu_menuitem_marshal_VOID__VOID,
 	                                           G_TYPE_NONE, 0, G_TYPE_NONE);
+	/**
+		DbusmenuMenuitem::show-to-user:
+		@arg0: The #DbusmenuMenuitem which should be shown.
+		@arg1: Timestamp the event happened at
+
+		Signaled when the application would like the visualization
+		of this menu item shown to the user.  This usually requires
+		going over the bus to get it done.
+	*/
+	signals[SHOW_TO_USER] =      g_signal_new(DBUSMENU_MENUITEM_SIGNAL_SHOW_TO_USER,
+	                                           G_TYPE_FROM_CLASS(klass),
+	                                           G_SIGNAL_RUN_LAST,
+	                                           G_STRUCT_OFFSET(DbusmenuMenuitemClass, show_to_user),
+	                                           NULL, NULL,
+	                                           g_cclosure_marshal_VOID__UINT,
+	                                           G_TYPE_NONE, 1, G_TYPE_UINT, G_TYPE_NONE);
 
 	g_object_class_install_property (object_class, PROP_ID,
 	                                 g_param_spec_int(PROP_ID_S, "ID for the menu item",
@@ -370,6 +389,28 @@ handle_event (DbusmenuMenuitem * mi, const gchar * name, const GValue * value, g
 	return;
 }
 
+/* Handles our about to show signal on items that submenus
+   exist.  This is sending just activate now, but we should
+   probably consider a special signal in the future if GTK
+   gets more sophisticated about this. */
+static void
+send_about_to_show (DbusmenuMenuitem * mi, void (*cb) (DbusmenuMenuitem * mi, gpointer user_data), gpointer cb_data)
+{
+	g_return_if_fail(DBUSMENU_IS_MENUITEM(mi));
+
+	if (dbusmenu_menuitem_get_children(mi) == NULL) {
+		g_warning("About to Show called on an item wihtout submenus.  We're ignoring it.");
+	} else {
+		g_signal_emit(G_OBJECT(mi), signals[ITEM_ACTIVATED], 0, 0 /* timestamp */, TRUE);
+	}
+
+	if (cb != NULL) {
+		cb(mi, cb_data);
+	}
+
+	return;
+}
+
 /* Public interface */
 
 /**
@@ -1333,7 +1374,7 @@ dbusmenu_menuitem_handle_event (DbusmenuMenuitem * mi, const gchar * name, const
 	called if possible.
 */
 void
-dbusmenu_menuitem_send_about_to_show (DbusmenuMenuitem * mi, dbusmenu_menuitem_about_to_show_cb cb, gpointer cb_data)
+dbusmenu_menuitem_send_about_to_show (DbusmenuMenuitem * mi, void (*cb) (DbusmenuMenuitem * mi, gpointer user_data), gpointer cb_data)
 {
 	g_return_if_fail(DBUSMENU_IS_MENUITEM(mi));
 	#ifdef MASSIVEDEBUGGING
@@ -1349,3 +1390,22 @@ dbusmenu_menuitem_send_about_to_show (DbusmenuMenuitem * mi, dbusmenu_menuitem_a
 
 	return;
 }
+
+/**
+	dbusmenu_menuitem_show_to_user:
+	@mi: #DbusmenuMenuitem to show
+	@timestamp: The time that the user requested it to be shown
+
+	Signals that this menu item should be shown to the user.  If this is
+	server side the server will then take it and send it over the
+	bus.
+*/
+void
+dbusmenu_menuitem_show_to_user (DbusmenuMenuitem * mi, guint timestamp)
+{
+	g_return_if_fail(DBUSMENU_IS_MENUITEM(mi));
+
+	g_signal_emit(G_OBJECT(mi), signals[SHOW_TO_USER], 0, timestamp, TRUE);
+
+	return;
+}
