@@ -54,6 +54,7 @@ static void new_menuitem (DbusmenuClient * client, DbusmenuMenuitem * mi, gpoint
 static void new_child (DbusmenuMenuitem * mi, DbusmenuMenuitem * child, guint position, DbusmenuGtkClient * gtkclient);
 static void delete_child (DbusmenuMenuitem * mi, DbusmenuMenuitem * child, DbusmenuGtkClient * gtkclient);
 static void move_child (DbusmenuMenuitem * mi, DbusmenuMenuitem * child, guint new, guint old, DbusmenuGtkClient * gtkclient);
+static void item_activate (DbusmenuClient * client, DbusmenuMenuitem * mi, guint timestamp, gpointer userdata);
 
 static gboolean new_item_normal     (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, DbusmenuClient * client);
 static gboolean new_item_seperator  (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, DbusmenuClient * client);
@@ -91,7 +92,9 @@ dbusmenu_gtkclient_init (DbusmenuGtkClient *self)
 	dbusmenu_client_add_type_handler(DBUSMENU_CLIENT(self), DBUSMENU_CLIENT_TYPES_DEFAULT,   new_item_normal);
 	dbusmenu_client_add_type_handler(DBUSMENU_CLIENT(self), DBUSMENU_CLIENT_TYPES_SEPARATOR, new_item_seperator);
 
+	/* TODO: I think these can be handled in the class... */
 	g_signal_connect(G_OBJECT(self), DBUSMENU_CLIENT_SIGNAL_NEW_MENUITEM, G_CALLBACK(new_menuitem), NULL);
+	g_signal_connect(G_OBJECT(self), DBUSMENU_CLIENT_SIGNAL_ITEM_ACTIVATE, G_CALLBACK(item_activate), NULL);
 
 	return;
 }
@@ -431,6 +434,57 @@ new_menuitem (DbusmenuClient * client, DbusmenuMenuitem * mi, gpointer userdata)
 	return;
 }
 
+/* Goes through the tree of items and ensure's that all the items
+   above us are also displayed. */
+static void
+activate_helper (GtkMenuShell * shell)
+{
+	if (shell == NULL) {
+		return;
+	}
+
+	if (GTK_IS_MENU(shell)) {
+		GtkWidget * attach = gtk_menu_get_attach_widget(GTK_MENU(shell));
+
+		if (attach != NULL) {
+			GtkWidget * parent = gtk_widget_get_parent(GTK_WIDGET(attach));
+
+			if (parent != NULL) {
+				if (GTK_IS_MENU(parent)) {
+					activate_helper(GTK_MENU_SHELL(parent));
+				}
+
+				if (!GTK_MENU_SHELL (parent)->active) {
+					gtk_grab_add (parent);
+					GTK_MENU_SHELL (parent)->have_grab = TRUE;
+					GTK_MENU_SHELL (parent)->active = TRUE;
+				}
+
+				gtk_menu_shell_select_item(GTK_MENU_SHELL(parent), attach);
+			}
+		}
+	}
+
+	return;
+}
+
+/* Signaled when we should show a menuitem at request of the application
+   that it is in. */
+static void
+item_activate (DbusmenuClient * client, DbusmenuMenuitem * mi, guint timestamp, gpointer userdata)
+{
+	gpointer pmenu = g_object_get_data(G_OBJECT(mi), data_menu);
+	if (pmenu == NULL) {
+		g_warning("Activated menu item doesn't have a menu?  ID: %d", dbusmenu_menuitem_get_id(mi));
+		return;
+	}
+
+	activate_helper(GTK_MENU_SHELL(pmenu));
+	gtk_menu_shell_select_first(GTK_MENU_SHELL(pmenu), FALSE);
+
+	return;
+}
+
 #ifdef MASSIVEDEBUGGING
 static void
 destroy_gmi (GtkMenuItem * gmi, DbusmenuMenuitem * mi)
